@@ -10,18 +10,19 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2018 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2006-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
  * Copyright (c) 2006-2009 University of Houston. All rights reserved.
  * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
  * Copyright (c) 2012-2013 Inria.  All rights reserved.
- * Copyright (c) 2014-2016 Intel, Inc. All rights reserved.
- * Copyright (c) 2014-2015 Research Organization for Information Science
+ * Copyright (c) 2014-2018 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2016 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2016      Mellanox Technologies Ltd. All rights reserved.
  *
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -34,9 +35,7 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif  /* HAVE_SYS_TIME_H */
-#ifdef HAVE_PTHREAD_H
 #include <pthread.h>
-#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -62,6 +61,7 @@
 #include "opal/mca/btl/base/base.h"
 #include "opal/mca/pmix/pmix.h"
 #include "opal/util/timings.h"
+#include "opal/util/opal_environ.h"
 
 #include "ompi/constants.h"
 #include "ompi/mpi/fortran/base/constants.h"
@@ -71,6 +71,7 @@
 #include "ompi/info/info.h"
 #include "ompi/errhandler/errcode.h"
 #include "ompi/errhandler/errhandler.h"
+#include "ompi/interlib/interlib.h"
 #include "ompi/request/request.h"
 #include "ompi/message/message.h"
 #include "ompi/op/op.h"
@@ -93,9 +94,17 @@
 #include "ompi/mca/pml/base/pml_base_bsend.h"
 #include "ompi/dpm/dpm.h"
 #include "ompi/mpiext/mpiext.h"
+#include "ompi/mca/hook/base/base.h"
+#include "ompi/util/timings.h"
+
+#if OPAL_ENABLE_FT_CR == 1
+#include "ompi/mca/crcp/crcp.h"
+#include "ompi/mca/crcp/base/base.h"
+#endif
+#include "ompi/runtime/ompi_cr.h"
 
 /* newer versions of gcc have poisoned this deprecated feature */
-#if HAVE___MALLOC_INITIALIZE_HOOK
+#ifdef HAVE___MALLOC_INITIALIZE_HOOK
 #include "opal/mca/memory/base/base.h"
 /* So this sucks, but with OPAL in its own library that is brought in
    implicity from libmpi, there are times when the malloc initialize
@@ -121,11 +130,7 @@ const char ompi_version_string[] = OMPI_IDENT_STRING;
  * Global variables and symbols for the MPI layer
  */
 
-opal_mutex_t ompi_mpi_bootstrap_mutex = OPAL_MUTEX_STATIC_INIT;
-volatile bool ompi_mpi_init_started = false;
-volatile bool ompi_mpi_initialized = false;
-volatile bool ompi_mpi_finalize_started = false;
-volatile bool ompi_mpi_finalized = false;
+volatile int32_t ompi_mpi_state = OMPI_MPI_STATE_NOT_INITIALIZED;
 volatile bool ompi_rte_initialized = false;
 
 bool ompi_mpi_thread_multiple = false;
@@ -273,7 +278,7 @@ opal_hash_table_t ompi_mpi_f90_complex_hashtable = {{0}};
  */
 opal_list_t ompi_registered_datareps = {{0}};
 
-bool ompi_enable_timing = false, ompi_enable_timing_ext = false;
+bool ompi_enable_timing = false;
 extern bool ompi_mpi_yield_when_idle;
 extern int ompi_mpi_event_tick_rate;
 
@@ -308,7 +313,6 @@ static int _convert_process_name_to_string(char** name_string,
     return ompi_rte_convert_process_name_to_string(name_string, name);
 }
 
-
 void ompi_mpi_thread_level(int requested, int *provided)
 {
     /**
@@ -322,15 +326,7 @@ void ompi_mpi_thread_level(int requested, int *provided)
      */
     ompi_mpi_thread_requested = requested;
 
-    if (OMPI_ENABLE_THREAD_MULTIPLE == 1) {
-        ompi_mpi_thread_provided = *provided = requested;
-    } else {
-        if (MPI_THREAD_MULTIPLE == requested) {
-            ompi_mpi_thread_provided = *provided = MPI_THREAD_SERIALIZED;
-        } else {
-            ompi_mpi_thread_provided = *provided = requested;
-        }
-    }
+    ompi_mpi_thread_provided = *provided = requested;
 
     if (!ompi_mpi_main_thread) {
         ompi_mpi_main_thread = opal_thread_get_self();
@@ -350,6 +346,9 @@ static int ompi_register_mca_variables(void)
     }
 
     /* check to see if we want timing information */
+    /* TODO: enable OMPI init and OMPI finalize timings if
+     * this variable was set to 1!
+     */
     ompi_enable_timing = false;
     (void) mca_base_var_register("ompi", "ompi", NULL, "timing",
                                  "Request that critical timing loops be measured",
@@ -358,13 +357,6 @@ static int ompi_register_mca_variables(void)
                                  MCA_BASE_VAR_SCOPE_READONLY,
                                  &ompi_enable_timing);
 
-    ompi_enable_timing_ext = false;
-    (void) mca_base_var_register("ompi", "ompi", NULL, "timing_ext",
-                                 "Request that critical timing loops be measured",
-                                 MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
-                                 OPAL_INFO_LVL_9,
-                                 MCA_BASE_VAR_SCOPE_READONLY,
-                                 &ompi_enable_timing_ext);
     return OMPI_SUCCESS;
 }
 
@@ -374,43 +366,52 @@ static void fence_release(int status, void *cbdata)
     *active = false;
 }
 
-int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
+int ompi_mpi_init(int argc, char **argv, int requested, int *provided,
+                  bool reinit_ok)
 {
     int ret;
     ompi_proc_t** procs;
     size_t nprocs;
     char *error = NULL;
-    char *cmd=NULL, *av=NULL;
+    ompi_errhandler_errtrk_t errtrk;
     volatile bool active;
-    OPAL_TIMING_DECLARE(tm);
-    OPAL_TIMING_INIT_EXT(&tm, OPAL_TIMING_GET_TIME_OF_DAY);
+    opal_list_t info;
+    opal_value_t *kv;
 
-    /* bitflag of the thread level support provided. To be used
-     * for the modex in order to work in heterogeneous environments. */
-    uint8_t threadlevel_bf;
+    OMPI_TIMING_INIT(32);
 
-    /* Ensure that we were not already initialized or finalized.
+    ompi_hook_base_mpi_init_top(argc, argv, requested, provided);
 
-       This lock is held for the duration of ompi_mpi_init() and
-       ompi_mpi_finalize().  Hence, if we get it, then no other thread
-       is inside the critical section (and we don't have to check the
-       *_started bool variables). */
-    opal_mutex_lock(&ompi_mpi_bootstrap_mutex);
-    if (ompi_mpi_finalized) {
-        opal_show_help("help-mpi-runtime.txt",
-                       "mpi_init: already finalized", true);
-        opal_mutex_unlock(&ompi_mpi_bootstrap_mutex);
-        return MPI_ERR_OTHER;
-    } else if (ompi_mpi_initialized) {
-        opal_show_help("help-mpi-runtime.txt",
-                       "mpi_init: invoked multiple times", true);
-        opal_mutex_unlock(&ompi_mpi_bootstrap_mutex);
-        return MPI_ERR_OTHER;
+    /* Ensure that we were not already initialized or finalized. */
+    int32_t expected = OMPI_MPI_STATE_NOT_INITIALIZED;
+    int32_t desired  = OMPI_MPI_STATE_INIT_STARTED;
+    opal_atomic_wmb();
+    if (!opal_atomic_cmpset_32(&ompi_mpi_state, expected, desired)) {
+        // If we failed to atomically transition ompi_mpi_state from
+        // NOT_INITIALIZED to INIT_STARTED, then someone else already
+        // did that, and we should return.
+        if (expected >= OMPI_MPI_STATE_FINALIZE_STARTED) {
+            opal_show_help("help-mpi-runtime.txt",
+                           "mpi_init: already finalized", true);
+            return MPI_ERR_OTHER;
+        } else if (expected >= OMPI_MPI_STATE_INIT_STARTED) {
+            // In some cases (e.g., oshmem_shmem_init()), we may call
+            // ompi_mpi_init() multiple times.  In such cases, just
+            // silently return successfully once the initializing
+            // thread has completed.
+            if (reinit_ok) {
+                while (ompi_mpi_state < OMPI_MPI_STATE_INIT_COMPLETED) {
+                    usleep(1);
+                }
+                return MPI_SUCCESS;
+            }
+
+            opal_show_help("help-mpi-runtime.txt",
+                           "mpi_init: invoked multiple times", true);
+            return MPI_ERR_OTHER;
+        }
     }
 
-    /* Indicate that we have *started* MPI_INIT* */
-    ompi_mpi_init_started = true;
-
     /* Figure out the final MPI thread levels.  If we were not
        compiled for support for MPI threads, then don't allow
        MPI_THREAD_MULTIPLE.  Set this stuff up here early in the
@@ -475,7 +476,16 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         mca_base_var_set_value(ret, allvalue, 4, MCA_BASE_VAR_SOURCE_DEFAULT, NULL);
     }
 
-    OPAL_TIMING_MSTART((&tm,"time from start to completion of rte_init"));
+    /* open the ompi hook framework */
+    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_hook_base_framework, 0))) {
+        error = "ompi_hook_base_open() failed";
+        goto error;
+    }
+
+    ompi_hook_base_mpi_init_top_post_opal(argc, argv, requested, provided);
+
+
+    OMPI_TIMING_NEXT("initialization");
 
     /* if we were not externally started, then we need to setup
      * some envars so the MPI_INFO_ENV can get the cmd name
@@ -483,15 +493,13 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
      * the requested thread level
      */
     if (NULL == getenv("OMPI_COMMAND") && NULL != argv && NULL != argv[0]) {
-        asprintf(&cmd, "OMPI_COMMAND=%s", argv[0]);
-        putenv(cmd);
+        opal_setenv("OMPI_COMMAND", argv[0], true, &environ);
     }
     if (NULL == getenv("OMPI_ARGV") && 1 < argc) {
         char *tmp;
         tmp = opal_argv_join(&argv[1], ' ');
-        asprintf(&av, "OMPI_ARGV=%s", tmp);
+        opal_setenv("OMPI_ARGV", tmp, true, &environ);
         free(tmp);
-        putenv(av);
     }
 
     /* open the rte framework */
@@ -506,41 +514,43 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         error = "ompi_mpi_init: ompi_rte_init failed";
         goto error;
     }
+
+    OMPI_TIMING_NEXT("rte_init");
+
     ompi_rte_initialized = true;
 
-    /* check for timing request - get stop time and report elapsed time if so */
-    OPAL_TIMING_MNEXT((&tm,"time from completion of rte_init to modex"));
+    /* Register the default errhandler callback  */
+    errtrk.status = OPAL_ERROR;
+    errtrk.active = true;
+    /* we want to go first */
+    OBJ_CONSTRUCT(&info, opal_list_t);
+    kv = OBJ_NEW(opal_value_t);
+    kv->key = strdup(OPAL_PMIX_EVENT_ORDER_PREPEND);
+    opal_list_append(&info, &kv->super);
+    /* give it a name so we can distinguish it */
+    kv = OBJ_NEW(opal_value_t);
+    kv->key = strdup(OPAL_PMIX_EVENT_HDLR_NAME);
+    kv->type = OPAL_STRING;
+    kv->data.string = strdup("MPI-Default");
+    opal_list_append(&info, &kv->super);
+    opal_pmix.register_evhandler(NULL, &info, ompi_errhandler_callback,
+                                 ompi_errhandler_registration_callback,
+                                 (void*)&errtrk);
+    OMPI_LAZY_WAIT_FOR_COMPLETION(errtrk.active);
 
-    /* if hwloc is available but didn't get setup for some
-     * reason, do so now
-     */
-    if (NULL == opal_hwloc_topology) {
-        if (OPAL_SUCCESS != (ret = opal_hwloc_base_get_topology())) {
-            error = "Topology init";
-            goto error;
-        }
+    OPAL_LIST_DESTRUCT(&info);
+    if (OPAL_SUCCESS != errtrk.status) {
+        error = "Error handler registration";
+        ret = errtrk.status;
+        goto error;
     }
 
-    /* Register the default errhandler callback - RTE will ignore if it
-     * doesn't support this capability
-     */
-    ompi_rte_register_errhandler(ompi_errhandler_runtime_callback,
-                                 OMPI_RTE_ERRHANDLER_LAST);
-
-
-    /* determine the bitflag belonging to the threadlevel_support provided */
-    memset ( &threadlevel_bf, 0, sizeof(uint8_t));
-    OMPI_THREADLEVEL_SET_BITFLAG ( ompi_mpi_thread_provided, threadlevel_bf );
-
-#if OMPI_ENABLE_THREAD_MULTIPLE
-    /* add this bitflag to the modex */
-    OPAL_MODEX_SEND_STRING(ret, OPAL_PMIX_GLOBAL,
-                           "MPI_THREAD_LEVEL", &threadlevel_bf, sizeof(uint8_t));
-    if (OPAL_SUCCESS != ret) {
-        error = "ompi_mpi_init: modex send thread level";
+    /* declare our presence for interlib coordination, and
+     * register for callbacks when other libs declare */
+    if (OMPI_SUCCESS != (ret = ompi_interlib_declare(*provided, OMPI_IDENT_STRING))) {
+        error = "ompi_interlib_declare";
         goto error;
     }
-#endif
 
     /* initialize datatypes. This step should be done early as it will
      * create the local convertor and local arch used in the proc
@@ -612,6 +622,13 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
+#if OPAL_ENABLE_FT_CR == 1
+    if (OMPI_SUCCESS != (ret = mca_base_framework_open(&ompi_crcp_base_framework, 0))) {
+        error = "ompi_crcp_base_open() failed";
+        goto error;
+    }
+#endif
+
     /* In order to reduce the common case for MPI apps (where they
        don't use MPI-2 IO or MPI-1 topology functions), the io and
        topo frameworks are initialized lazily, at the first use of
@@ -627,26 +644,55 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
-    /* check for timing request - get stop time and report elapsed time if so */
-    OPAL_TIMING_MNEXT((&tm,"time to execute modex"));
+    OMPI_TIMING_IMPORT_OPAL("orte_init");
+    OMPI_TIMING_IMPORT_OPAL("opal_init_util");
+    OMPI_TIMING_NEXT("rte_init-commit");
+
 
     /* exchange connection info - this function may also act as a barrier
      * if data exchange is required. The modex occurs solely across procs
      * in our job. If a barrier is required, the "modex" function will
      * perform it internally */
-    active = true;
     opal_pmix.commit();
-    if (!opal_pmix_base_async_modex) {
-        if (NULL != opal_pmix.fence_nb) {
+    OMPI_TIMING_NEXT("commit");
+
+    /* If we have a non-blocking fence:
+     * if we are doing an async modex, but we are collecting all
+     * data, then execute the non-blocking modex in the background.
+     * All calls to modex_recv will be cached until the background
+     * modex completes. If collect_all_data is false, then we skip
+     * the fence completely and retrieve data on-demand from the
+     * source node.
+     *
+     * If we do not have a non-blocking fence, then we must always
+     * execute the blocking fence as the system does not support
+     * later data retrieval. */
+    if (NULL != opal_pmix.fence_nb) {
+        if (opal_pmix_base_async_modex && opal_pmix_collect_all_data) {
+            /* execute the fence_nb in the background to collect
+             * the data */
+            if (!ompi_async_mpi_init) {
+                /* we are going to execute a barrier at the
+                 * end of MPI_Init. We can only have ONE fence
+                 * operation with the identical involved procs
+                 * at a time, so we will need to wait when we
+                 * get there */
+                active = true;
+                opal_pmix.fence_nb(NULL, true, fence_release, (void*)&active);
+            } else {
+                opal_pmix.fence_nb(NULL, true, NULL, NULL);
+            }
+        } else if (!opal_pmix_base_async_modex) {
+            active = true;
             opal_pmix.fence_nb(NULL, opal_pmix_collect_all_data,
                                fence_release, (void*)&active);
             OMPI_LAZY_WAIT_FOR_COMPLETION(active);
-        } else {
-            opal_pmix.fence(NULL, opal_pmix_collect_all_data);
         }
+    } else {
+        opal_pmix.fence(NULL, opal_pmix_collect_all_data);
     }
 
-    OPAL_TIMING_MNEXT((&tm,"time from modex to first barrier"));
+    OMPI_TIMING_NEXT("modex");
 
     /* select buffered send allocator component to be used */
     if( OMPI_SUCCESS !=
@@ -669,6 +715,13 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
+#if OPAL_ENABLE_FT_CR == 1
+    if (OMPI_SUCCESS != (ret = ompi_crcp_base_select() ) ) {
+        error = "ompi_crcp_base_select() failed";
+        goto error;
+    }
+#endif
+
     /* io and topo components are not selected here -- see comment
        above about the io and topo frameworks being loaded lazily */
 
@@ -685,7 +738,7 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
     }
 
     /* initialize info */
-    if (OMPI_SUCCESS != (ret = ompi_info_init())) {
+    if (OMPI_SUCCESS != (ret = ompi_mpiinfo_init())) {
         error = "ompi_info_init() failed";
         goto error;
     }
@@ -800,24 +853,32 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
     ompi_rte_wait_for_debugger();
 
     /* Next timing measurement */
-    OPAL_TIMING_MNEXT((&tm,"time to execute barrier"));
+    OMPI_TIMING_NEXT("modex-barrier");
 
     /* wait for everyone to reach this point - this is a hard
      * barrier requirement at this time, though we hope to relax
      * it at a later point */
-    active = true;
-    opal_pmix.commit();
-    if (NULL != opal_pmix.fence_nb) {
-        opal_pmix.fence_nb(NULL, false,
-                           fence_release, (void*)&active);
-        OMPI_LAZY_WAIT_FOR_COMPLETION(active);
-    } else {
-        opal_pmix.fence(NULL, false);
+    if (!ompi_async_mpi_init) {
+        /* if we executed the above fence in the background, then
+         * we have to wait here for it to complete. However, there
+         * is no reason to do two barriers! */
+        if (opal_pmix_base_async_modex && opal_pmix_collect_all_data) {
+            OMPI_LAZY_WAIT_FOR_COMPLETION(active);
+        } else {
+            active = true;
+            if (NULL != opal_pmix.fence_nb) {
+                opal_pmix.fence_nb(NULL, false,
+                                   fence_release, (void*)&active);
+                OMPI_LAZY_WAIT_FOR_COMPLETION(active);
+            } else {
+                opal_pmix.fence(NULL, false);
+            }
+        }
     }
 
     /* check for timing request - get stop time and report elapsed
        time if so, then start the clock again */
-    OPAL_TIMING_MNEXT((&tm,"time from barrier to complete mpi_init"));
+    OMPI_TIMING_NEXT("barrier");
 
 #if OPAL_ENABLE_PROGRESS_THREADS == 0
     /* Start setting up the event engine for MPI operations.  Don't
@@ -879,6 +940,16 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
         goto error;
     }
 
+    /*
+     * Startup the Checkpoint/Restart Mech.
+     * Note: Always do this so tools don't hang when
+     * in a non-checkpointable build
+     */
+    if (OMPI_SUCCESS != (ret = ompi_cr_init())) {
+        error = "ompi_cr_init";
+        goto error;
+    }
+
     /* Undo OPAL calling opal_progress_event_users_increment() during
        opal_init, to get better latency when not using TCP.  Do
        this *after* dyn_init, as dyn init uses lots of RTE
@@ -908,13 +979,14 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
  error:
     if (ret != OMPI_SUCCESS) {
         /* Only print a message if one was not already printed */
-        if (NULL != error) {
+        if (NULL != error && OMPI_ERR_SILENT != ret) {
             const char *err_msg = opal_strerror(ret);
             opal_show_help("help-mpi-runtime.txt",
                            "mpi_init:startup:internal-failure", true,
                            "MPI_INIT", "MPI_INIT", error, err_msg, ret);
         }
-        opal_mutex_unlock(&ompi_mpi_bootstrap_mutex);
+        ompi_hook_base_mpi_init_error(argc, argv, requested, provided);
+        OMPI_TIMING_FINALIZE;
         return ret;
     }
 
@@ -934,16 +1006,16 @@ int ompi_mpi_init(int argc, char **argv, int requested, int *provided)
     opal_hash_table_init(&ompi_mpi_f90_complex_hashtable, FLT_MAX_10_EXP);
 
     /* All done.  Wasn't that simple? */
-
-    ompi_mpi_initialized = true;
+    opal_atomic_wmb();
+    opal_atomic_swap_32(&ompi_mpi_state, OMPI_MPI_STATE_INIT_COMPLETED);
 
     /* Finish last measurement, output results
      * and clear timing structure */
-    OPAL_TIMING_MSTOP(&tm);
-    OPAL_TIMING_DELTAS(ompi_enable_timing, &tm);
-    OPAL_TIMING_REPORT(ompi_enable_timing_ext, &tm);
-    OPAL_TIMING_RELEASE(&tm);
+    OMPI_TIMING_NEXT("barrier-finish");
+    OMPI_TIMING_OUT;
+    OMPI_TIMING_FINALIZE;
+
+    ompi_hook_base_mpi_init_bottom(argc, argv, requested, provided);
 
-    opal_mutex_unlock(&ompi_mpi_bootstrap_mutex);
     return MPI_SUCCESS;
 }
