@@ -248,7 +248,7 @@ static inline opal_free_list_item_t *opal_free_list_get (opal_free_list_t *flist
 static inline opal_free_list_item_t *opal_free_list_wait_mt (opal_free_list_t *fl)
 {
     opal_free_list_item_t *item =
-        (opal_free_list_item_t *) opal_lifo_pop (&fl->super);
+        (opal_free_list_item_t *) opal_lifo_pop_atomic (&fl->super);
 
     while (NULL == item) {
         if (!opal_mutex_trylock (&fl->fl_lock)) {
@@ -274,7 +274,7 @@ static inline opal_free_list_item_t *opal_free_list_wait_mt (opal_free_list_t *f
             opal_mutex_lock (&fl->fl_lock);
         }
         opal_mutex_unlock (&fl->fl_lock);
-        item = (opal_free_list_item_t *) opal_lifo_pop (&fl->super);
+        item = (opal_free_list_item_t *) opal_lifo_pop_atomic (&fl->super);
     }
 
     return item;
