@@ -12,7 +12,7 @@
  * Copyright (c) 2006-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2014-2015 Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2017 Intel, Inc.  All rights reserved.
  * Copyright (c) 2014-2015 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -149,7 +149,7 @@ static int orte_rmaps_base_register(mca_base_register_flag_t flags)
                                  MCA_BASE_VAR_SCOPE_READONLY, &rmaps_base_bynode);
 
     /* #cpus/rank to use */
-    orte_rmaps_base.cpus_per_rank = 1;
+    orte_rmaps_base.cpus_per_rank = 0;
     var_id = mca_base_var_register("orte", "rmaps", "base", "cpus_per_proc",
                                    "Number of cpus to use for each rank [1-2**15 (default=1)]",
                                    MCA_BASE_VAR_TYPE_INT, NULL, 0, 0,
@@ -253,6 +253,7 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
     orte_rmaps_base.slot_list = NULL;
     orte_rmaps_base.mapping = 0;
     orte_rmaps_base.ranking = 0;
+    orte_rmaps_base.device = NULL;
 
     /* if a topology file was given, then set our topology
      * from it. Even though our actual topology may differ,
@@ -279,7 +280,7 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
             return ORTE_ERR_SILENT;
         }
     }
-    if (1 < orte_rmaps_base.cpus_per_rank) {
+    if (0 < orte_rmaps_base.cpus_per_rank) {
         orte_show_help("help-orte-rmaps-base.txt", "deprecated", true,
                        "--cpus-per-proc, -cpus-per-proc, --cpus-per-rank, -cpus-per-rank",
                        "--map-by <obj>:PE=N, default <obj>=NUMA",
@@ -375,8 +376,8 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
         ORTE_SET_RANKING_DIRECTIVE(orte_rmaps_base.ranking, ORTE_RANKING_GIVEN);
     }
 
-    if (1 < orte_rmaps_base.cpus_per_rank) {
-        /* if we were asked for multiple cpus/proc, then we have to
+    if (0 < orte_rmaps_base.cpus_per_rank) {
+        /* if we were asked for cpus/proc, then we have to
          * bind to those cpus - any other binding policy is an
          * error
          */
@@ -405,21 +406,23 @@ static int orte_rmaps_base_open(mca_base_open_flag_t flags)
                 OPAL_SET_BINDING_POLICY(opal_hwloc_binding_policy, OPAL_BIND_TO_CORE);
             }
         }
-        /* we also need to ensure we are mapping to a high-enough level to have
-         * multiple cpus beneath it - by default, we'll go to the NUMA level */
-        if (ORTE_MAPPING_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping)) {
-            if (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYHWTHREAD ||
-              (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYCORE &&
-              !opal_hwloc_use_hwthreads_as_cpus)) {
-                orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low-init", true);
-                return ORTE_ERR_SILENT;
+        if (1 < orte_rmaps_base.cpus_per_rank) {
+            /* we need to ensure we are mapping to a high-enough level to have
+             * multiple cpus beneath it - by default, we'll go to the NUMA level */
+            if (ORTE_MAPPING_GIVEN & ORTE_GET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping)) {
+                if (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYHWTHREAD ||
+                  (ORTE_GET_MAPPING_POLICY(orte_rmaps_base.mapping) == ORTE_MAPPING_BYCORE &&
+                  !opal_hwloc_use_hwthreads_as_cpus)) {
+                    orte_show_help("help-orte-rmaps-base.txt", "mapping-too-low-init", true);
+                    return ORTE_ERR_SILENT;
+                }
+            } else {
+                opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
+                                    "%s rmaps:base pe/rank set - setting mapping to BYNUMA",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
+                ORTE_SET_MAPPING_POLICY(orte_rmaps_base.mapping, ORTE_MAPPING_BYNUMA);
+                ORTE_SET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_GIVEN);
             }
-        } else {
-            opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                                "%s rmaps:base pe/rank set - setting mapping to BYNUMA",
-                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME));
-            ORTE_SET_MAPPING_POLICY(orte_rmaps_base.mapping, ORTE_MAPPING_BYNUMA);
-            ORTE_SET_MAPPING_DIRECTIVE(orte_rmaps_base.mapping, ORTE_MAPPING_GIVEN);
         }
     }
 
@@ -614,9 +617,10 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
     }
 
     opal_output_verbose(5, orte_rmaps_base_framework.framework_output,
-                        "%s rmaps:base set policy with %s",
+                        "%s rmaps:base set policy with %s device %s",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                        (NULL == inspec) ? "NULL" : inspec);
+                        (NULL == inspec) ? "NULL" : inspec,
+                        (NULL == device) ? "NULL" : "NONNULL");
 
     if (NULL == inspec) {
         ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYSOCKET);
@@ -719,12 +723,14 @@ int orte_rmaps_base_set_mapping_policy(orte_mapping_policy_t *policy,
              * we need to treat those hwthreads as separate cpus
              */
             opal_hwloc_use_hwthreads_as_cpus = true;
-        } else if ( NULL != device && 0 == strncasecmp(spec, "dist", len)) {
+        } else if (0 == strncasecmp(spec, "dist", len)) {
             if (NULL != rmaps_dist_device) {
                 if (NULL != (pch = strchr(rmaps_dist_device, ':'))) {
                     *pch = '\0';
                 }
-                *device = strdup(rmaps_dist_device);
+                if (NULL != device) {
+                    *device = strdup(rmaps_dist_device);
+                }
                 ORTE_SET_MAPPING_POLICY(tmp, ORTE_MAPPING_BYDIST);
             } else {
                 orte_show_help("help-orte-rmaps-base.txt", "device-not-specified", true);
